-- browser.applescript
-- iGP32

--  Created by Kounch on Tue Jul 13 2004.
--  Copyright (c) 2003 Kounch. All rights reserved.

-- This file is part of iGP32.
-- 
--  iGP32 is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.
--
--  iGP32 is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with iGP32; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(* Based on Apple example which demonstrates how to script a browser object. The  parts of this browsing code are the "number of browser rows" event handler which needs to return number of rows in the browser for the given column, and the "will display browser cell" event handler that will be called for every item in the browser. *)

(* ==== Properties ==== *)
global thePath
global mainWindow
global browserWindow
global progressPanel
global browserControl
global destDir
global destDirText
global context_menu
global listam
global panelOpen
global panelClick
global SMCSize
global smcFree
global maclinkLib
global maclinkPath

property diskNames : {}
property USBDelay : 0.1 --Delay after using maclink. Will this stop maclink hang ups?
property fatalError : false

--Initialization when opening browser window
on awake from nib theObject
	set object_name to name of theObject as string
	if object_name = "browser" then
		--Register browser for drag and drop 
		tell theObject to register drag types {"file names"}
	else if object_name = "contextual" then
		--Get reference to menu so we can asign it later to cells of browser object
		set context_menu to theObject
	end if
end awake from nib

--Button click event handler
on clicked theObject
	if the name of theObject is "browse" then
		--The user clicks on browser button of main window
		initBrowser()
	else if the name of theObject is "delete" then
		deleteItem()
	else if the name of theObject is "mkdir" then
		makeDirectory()
	else if the name of theObject is "download" then
		Downloadselection()
	end if
end clicked

--Browser window initalizing code
on initBrowser()
	--Load external library of functions
	try
		set scriptpath to ((path to me) & "Contents:Resources:Scripts:maclink.scpt") as string
		set maclinkLib to (load script file scriptpath)
		initLib() of maclinkLib
	on error errTexto
		--Normally an error means that other proccess is using the maclinklib, so we beep and return
		beep
		log errTexto
		return
	end try
	
	--Define browser window
	set browserWindow to window "browserWindow"
	--Define browser control
	set browserControl to browser "browser" of browserWindow
	--Define text field
	set destDirText to "destDir"
	
	--refresh browser control
	updateBrowser()
end initBrowser


--Browser control refresh code
on updateBrowser()
	--Get list of files and folders in root directory of SMC
	try
		delay USBDelay
		set theResult to ""
		set theResult to do shell script maclinkPath & " ls /"
	on error
		log "maclink ls Error: " & theResult
		--If we get in here, there has been some problem with USB connection or the browser window
		USBError() of maclinkLib
		--Close browser window as the data it is showing could be wrong or incomplete
		hide browserWindow
		set fatalError to true
		return
	end try
	
	if (count of paragraphs of theResult) < 3 then
		log "maclink ls Data Error: " & theResult
		--If we get in here, there has been some problem with USB connection
		USBError() of maclinkLib
		--Close browser window as the data it is showing could be wrong or incomplete
		hide browserWindow
		set fatalError to true
		return
	else if (count of paragraphs of theResult) < 4 then
		--Card completely empty, no directory structure
		--So we warn the user that the card must be initialized
		set confirmText to localized string "CONFIRM_KEY" from table "Localized"
		set formatText to localized string "PLEASE_FORMAT_KEY" from table "Localized"
		set cancelText to localized string "CANCEL_KEY" from table "Localized"
		--Show error dialog
		set withResult to display alert confirmText as critical message formatText default button cancelText
		--close window
		hide browserWindow
	else
		--Initialize browser
		--diskNames stores the contents of the root directory of SMC
		set diskNames to paragraphs 4 thru (count of paragraphs of theResult) of theResult as list
		
		try
			--try to connect to gp32 device and obtain data
			delay USBDelay
			set theResult to ""
			set theInfo to do shell script maclinkPath & " info"
		on error
			log "maclink info Error: " & theResult
			--If we get in here, there has been some problem with USB connection or the browser window
			USBError() of maclinkLib
			--Close browser window as the data it is showing could be wrong or incomplete
			hide browserWindow
			set fatalError to true
			return
		end try
		
		--Get info about SMC
		set SMCSize to (get word 6 of paragraph 8 of theInfo)
		set smcFree to (get word 8 of paragraph 8 of theInfo)
		--Update browser window with SMC info
		set textFree to localized string "SMC_FREE_KEY" from table "Localized"
		tell browserWindow
			set contents of text field "free" to (((smcFree div 1024) as string) & textFree)
			update
		end tell
		--Show  browser window
		show browserWindow
		
		--initialize browser object
		set path separator of browserControl to "/"
		--Update browser to show root directory
		tell browserControl to update
	end if
end updateBrowser

-- Return the number of rows for the given column
on number of browser rows theObject in column theColumn
	set rowCount to 0
	--Check if there is at least one item on root directory
	if (count of diskNames) > 0 then
		if theColumn is 1 then
			--If it is the root directory, we can already count the number of items
			set rowCount to count of diskNames
		else
			--get Path to previous item in tree file hierarchy
			tell theObject
				set thePath to path for column theColumn - 1
			end tell
			--Try to list directory
			try
				--Connect to GP32 and list files
				delay USBDelay
				set theResult to ""
				set theResult to do shell script maclinkPath & " ls /" & (thePath as string)
				--We trim the first three paragraphs are copyright strings and other data and get the number of items
				set rowCount to (count of paragraphs of theResult) - 3
				--We also cut only the paragraphs we need and convert the path from text to a list
				set thePath to paragraphs 4 thru (count of paragraphs of theResult) of theResult as list
			on error
				log "maclink ls Error: " & theResult
				--If we get in here, there has been some problem with USB connection or the browser window
				USBError() of maclinkLib
				--Close browser window as the data it is showing could be wrong or incomplete
				hide browserWindow
			end try
		end if
		
		--After obtaining the list of items in the directory we check browser control selected items
		tell browserControl
			--path of selected column
			set destDir to path for column (selected column)
			--is any cell selected?
			if (count of selected cell) > 0 then
				--is the selected cell a directory?
				if (leaf of selected cell) = 0 then
					--append directory name to path 
					set destDir to destDir & "/" & (contents of selected cell)
				end if
			end if
			--mark the destination as a directory
			set destDir to destDir & "/"
		end tell
		
		--Update browserWindow
		tell browserWindow
			set contents of text field destDirText to ("gp:" & destDir)
		end tell
		
	end if
	return rowCount
end number of browser rows


-- This is called whenever a cell in the browser needs to be displayed.
on will display browser cell theObject row theRow browser cell theCell in column theColumn
	--Get contents of cell
	if theColumn is 1 then
		--We are updating the root directory
		set cellContents to item theRow of diskNames as string
	else
		--We are updating a subdirectory
		set cellContents to item theRow of thePath as string
	end if
	
	--Update values of one element in browser column
	if last character of cellContents is equal to "/" then
		--it is a folder
		set leaf of theCell to false
		--set cell image
		set artImage to load image "folder"
		set image of theCell to artImage
		--cut off "/" character of ending of displayed name of folder
		set numer to count characters of cellContents
		set string value of theCell to (get text 1 thru (numer - 1) of cellContents as string)
	else
		--it is a file
		set leaf of theCell to true
		set string value of theCell to cellContents
	end if
	
	---Register contextual menu for cell element
	set menu of theCell to context_menu
	--set Script of (menu of theCell) to me
end will display browser cell


--Drop event handler
on drop theObject drag info dragInfo
	--get types of data from the dragged object(s)
	set dataTypes to types of pasteboard of dragInfo
	--flag to determine success of drop action
	set dropped to false
	--Check if the dragged items are files
	if "file names" is in dataTypes then
		-- Initialize the list of files to an empty list
		set theFiles to {}
		-- We want the data as a list of file names, so set the preferred type to "file names"
		set preferred type of pasteboard of dragInfo to "file names"
		-- Get the list of files from the pasteboard
		set theFiles to contents of pasteboard of dragInfo
		--display progress panel
		display progressPanel attached to browserWindow
		tell progressPanel
			set actionText to localized string "UPLOAD_KEY" from table "Localized"
			set contents of text field "action" to actionText
			start progress indicator "progress"
		end tell
		
		set fatalError to false
		--Recursively copy files and folders
		CopyToGP32(theFiles, destDir)
		
		--close panel
		close panel (progressPanel)
		--drop operation was successful
		set dropped to true
		
		--Refresh browser
		updateBrowser()
	end if
	-- Set the preferred type back to the default
	set preferred type of pasteboard of dragInfo to ""
	--Return status of drag operation
	return dropped
end drop


--Handler which selects browser object when entering with a dragged object
on drag entered theObject drag info dragInfo
	if theObject is browserControl then
		set first responder of browserWindow to theObject
	end if
end drag entered


--Handler which deselects browser object when leaving with a dragged object
on drag exited theObject drag info dragInfo
	set first responder of browserWindow to browserWindow
end drag exited

--Handler called before closing the browser window
on will close theObject
	--Try to end PC-Link mode
	try
		delay USBDelay
		set theResult to ""
		set theResult to do shell script maclinkPath & " end"
	on error
		log "maclink end Error: " & theResult
		--If we get in here, there has been some problem with USB connection or the browser window
		USBError() of maclinkLib
	end try
	
end will close


--Handler called to copy files and folders to SMC of GP32
on CopyToGP32(theFiles, destPath)
	-- For every item in the list, try to copy into the GP32 device
	repeat with theItem in theFiles
		try
			--determine the kind of file theItem
			set theResult to ""
			set theResult to do shell script "file '" & theItem & "' | awk -F ':' '{print $2}'"
			--Variable to detect directories
			if theResult is not equal to " directory" then
				set isDirectory to false
			else
				set isDirectory to true
			end if
		on error errText number errNumber
			log "file command Error: " & theResult
			--If we get in here, there has been some problem with the command to analyze the file "theItem"
			set errorText to localized string "ERR_KEY" from table "Localized"
			set okText to localized string "OK_KEY" from table "Localized"
			display alert errorText as critical message errText default button okText
		end try
		
		--Keep AppleScript's text items delimiters values
		set savedTextItemDelimiters to AppleScript's text item delimiters
		--Get name of file
		set AppleScript's text item delimiters to {"/"}
		set fileName to last text item of (theItem as string)
		set AppleScript's text item delimiters to savedTextItemDelimiters
		
		--check if file name is in 8.3 format
		set nameOK to (checkFileName(fileName) of maclinkLib)
		
		--copy file if it is in 8.3 format
		if nameOK then
			--Display file name in panel
			tell progressPanel
				set contents of text field "filename" to fileName
				update
			end tell
			--Check if there is enough free space to copy file theItem
			set theResult to do shell script "du -ck " & theItem
			set fileSize to (first word of last paragraph of theResult) * 1024
			
			if fileSize < smcFree then
				--Try to copy the Item to GP32
				
				--check if it is a file or a directory
				if not isDirectory then
					--Try to copy file theItem which was dropped to theObject browser, to GP32
					Upload(theItem, destPath) of maclinkLib
					if fatalError is true then
						log "Fatal Error after Uploading " & theItem
						--Close browser window as the data it is showing could be wrong or incomplete
						hide browserWindow
					end if
				else
					--dragged object is a directory
					set newPath to (destPath as string) & (fileName as string) & "/"
					try
						--Create directory in GP32
						delay USBDelay
						set theResult to do shell script maclinkPath & " mkdir " & newPath
					end try
					--Get directory listing, convert it to a list of paths 
					try
						set theResult to do shell script " ls \"" & (theItem as string) & "\""
						--We also cut only the paragraphs we need and convert the path from text to a list
						set newfiles to {}
						repeat with n from 1 to count of paragraphs of theResult
							--Add path to all files in the list
							set newfiles to newfiles & ((theItem as string) & "/" & (paragraph n of theResult)) as list
						end repeat
					end try
					--Finally, call recursively this function copyToGP32 again with the contents of the directory
					CopyToGP32(newfiles, newPath)
				end if
			else
				--Inform user that there is not enough space
				set errorText to localized string "ERR_KEY" from table "Localized"
				set errSpace to localized string "ERR_SPACE_KEY" from table "Localized"
				set okText to localized string "OK_KEY" from table "Localized"
				display alert errorText as critical message (errSpace & fileName) default button okText
			end if
		else
			FileNameError() of maclinkLib
		end if
	end repeat
end CopyToGP32


--Handler to delete items
on deleteItem()
	tell browserControl
		if (count of selected cell) > 0 then
			set isSelected to true
			--Get info about what we have to delete
			set itemName to string value of selected cell
			set itemPath to path for column (selected column)
			set isDirectory to leaf of selected cell
		else
			set isSelected to false
		end if
	end tell
	
	if isSelected is true then
		--Make sure that the user wants to delete it
		set confirmText to localized string "CONFIRM_KEY" from table "Localized"
		set formatText to (localized string "ASK_DELETE_KEY" from table "Localized") & itemName
		set okText to localized string "OK_KEY" from table "Localized"
		set cancelText to localized string "CANCEL_KEY" from table "Localized"
		--Show warning dialog
		set panelOpen to true
		set withResult to display alert confirmText as warning message formatText default button cancelText alternate button okText attached to browserWindow
		
		repeat while panelOpen
			--wait till the dialog is dismissed
			delay 1
		end repeat
		
		--Check if the user has decided to delete the item
		if button returned of panelClick is equal to okText then
			--Determine command depending on kind of cell
			if isDirectory = 0 then
				set commandline to " rmdir \"/"
			else
				set commandline to " rm \"/"
			end if
			
			try
				--Delete item in SMC
				delay USBDelay
				set theResult to ""
				set theResult to do shell script maclinkPath & commandline & (itemPath as string) & "/" & (itemName as string) & "\""
			on error
				log "maclink rm or rmdir Error: " & theResult
				--If we get in here, there has been some problem with USB connection or the browser window
				USBError() of maclinkLib
				--Close browser window as the data it is showing could be wrong or incomplete
				hide browserWindow
			end try
			--Refresh browser
			updateBrowser()
		end if
	else
		--Nothing selected
	end if
end deleteItem

--Handler to create a directory in the SMC
on makeDirectory()
	--get path to selected cell
	tell browserControl
		if (count of selected cell) > 0 then
			--Get info about selected cell
			set itemName to string value of selected cell
			set itemPath to "/" & (path for column (selected column))
			if (leaf of selected cell) = 0 then
				set itemPath to itemPath & "/" & itemName
			end if
		else
			set itemPath to "/"
		end if
	end tell
	
	set askText to localized string "ASK_DIRECTORY" from table "Localized"
	set okText to localized string "OK_KEY" from table "Localized"
	set cancelText to localized string "CANCEL_KEY" from table "Localized"
	
	set panelOpen to true
	display dialog askText default answer "" buttons {okText, cancelText} default button okText attached to browserWindow
	repeat while panelOpen
		--wait till the dialog is dismissed
		delay 1
	end repeat
	
	--Check if the user has decided to create a new directory
	if button returned of panelClick is equal to okText then
		set itemName to (text returned of panelClick)
		--check if item name is in 8.3 format
		set nameOK to (checkFileName(itemName) of maclinkLib)
		
		--copy file if it is in 8.3 format
		if nameOK then
			try
				--Create directory in SMC
				delay USBDelay
				set theResult to ""
				set theResult to do shell script maclinkPath & " mkdir " & itemPath & "/" & itemName
			on error
				log "maclink mkdir Error: " & theResult
				--If we get in here, there has been some problem with USB connection or the browser window
				USBError() of maclinkLib
				--Close browser window as the data it is showing could be wrong or incomplete
				hide browserWindow
			end try
			--Refresh browser
			updateBrowser()
		else
			--Display an error when the file name is not in 8.3 format
			set errorText to localized string "ERR_KEY" from table "Localized"
			set errDOSText to localized string "ERR_DOSDIR_KEY" from table "Localized"
			set okText to localized string "OK_KEY" from table "Localized"
			display alert errorText as warning message errDOSText default button okText attached to browserWindow
		end if
	end if
end makeDirectory


--Handler to download files to the maclink
on Downloadselection()
	tell browserControl
		if (count of selected cell) > 0 then
			set isSelected to true
			--Get info about what we have to copy
			set itemName to string value of selected cell
			set itemPath to path for column (selected column)
			set isDirectory to leaf of selected cell
			
			set fullPath to itemPath & "/" & itemName
			--Mark as directory if necessary
			if isDirectory = 0 then
				set fullPath to fullPath & "/"
			end if
			--Initialize list of items to copy
			set theFiles to fullPath as list
		else
			set isSelected to false
		end if
	end tell
	
	if isSelected is true then
		--Define directory select panel to define destination path for selected files
		set can choose directories of open panel to true
		set can choose files of open panel to false
		set selectTitle to localized string "DOWNLOAD_DEST_TITLE_KEY" from table "Localized"
		set title of open panel to selectTitle
		set selectButton to localized string "SAVE_BUTTON_KEY" from table "Localized"
		set prompt of open panel to selectButton
		--Show panel
		set withResult to display open panel
		
		--Check if a directory was selected
		if withResult is 1 then
			--Define destination directory
			set downloadDir to (path names of open panel as list) as string
			
			tell progressPanel
				set actionText to localized string "DOWNLOAD_KEY" from table "Localized"
				set contents of text field "action" to actionText
				start progress indicator "progress"
			end tell
			display progressPanel attached to browserWindow
			
			--Download items from gp32
			CopyFromGP32(theFiles, downloadDir)
			
			--close panel
			close panel (progressPanel)
		end if
		
	end if
end Downloadselection


--Handler called to copy files and folders from SMC of GP32
on CopyFromGP32(theFiles, destPath)
	-- For every item in the list, try to copy from the GP32 device
	repeat with theItem in theFiles
		--determine the kind of theItem
		if last character of theItem is "/" then
			set isDirectory to true
		else
			set isDirectory to false
		end if
		
		set savedTextItemDelimiters to AppleScript's text item delimiters
		--Get name of file
		set AppleScript's text item delimiters to {"/"}
		if not isDirectory then
			set fileName to last text item of (theItem as string)
		else
			set penultimo to ((count of text items of (theItem as string)) - 1)
			set fileName to text item penultimo of (theItem as string)
		end if
		set AppleScript's text item delimiters to savedTextItemDelimiters
		
		--check if it is a file or a directory
		if not isDirectory then
			--Display item name in panel
			tell progressPanel
				set contents of text field "filename" to fileName
				update
			end tell
			--Try to copy file theItem from GP32
			Download(theItem, destPath)
		else
			--object is a directory
			try
				--create dir on destination
				set newPath to destPath & "/" & fileName
				do shell script "mkdir \"" & newPath & "\""
			end try
			
			try
				--Connect to GP32 and list files
				delay USBDelay
				set theResult to ""
				set theResult to do shell script maclinkPath & " ls /" & (theItem as string)
				--We trim the first three paragraphs which are copyright strings and other data 
				set newfiles to {}
				repeat with n from 4 to count of paragraphs of theResult
					--Add path to all files in the list
					set newfiles to newfiles & ((theItem as string) & (paragraph n of theResult)) as list
				end repeat
			on error
				log "maclink ls Error: " & theResult
				--If we get in here, there has been some problem with USB connection or the browser window
				USBError() of maclinkLib
				--Close browser window as the data it is showing could be wrong or incomplete
				hide browserWindow
			end try
			--Call this function recursively to copy all files in path
			CopyFromGP32(newfiles, newPath)
		end if
	end repeat
end CopyFromGP32


--Handler called to download theItem file to the GP32, after dropped to browser theObject
on Download(theItem, destPath)
	try
		--Download file from gp32 device
		delay USBDelay
		set theResult to ""
		set theResult to do shell script maclinkPath & " dl \"" & (destPath as string) & "\" \"" & (theItem as string) & "\""
	on error
		log "maclink dl Error: " & theResult
		--close panel
		close panel (progressPanel)
		--If we get in here, there has been some problem with USB connection or the browser window
		USBError() of maclinkLib
		--Close browser window as the data it is showing could be wrong or incomplete
		hide browserWindow
	end try
end Download


--Handler called when a panel is dismissed
on panel ended theObject with result withResult
	set panelOpen to false
	set panelClick to withResult
end panel ended

on dialog ended theObject with reply withReply
	set panelOpen to false
	set panelClick to withReply
end dialog ended

on alert ended theObject with reply withReply
	set panelClick to withReply
	set panelOpen to false
end alert ended

--Browser window remembering position handlers
on moved theObject
	--Get window coordinates
	set {windowX, windowY} to position of theObject
	--Save coordinates to defaults
	set contents of default entry "defaultBrowserWindowX" of user defaults to windowX
	set contents of default entry "defaultBrowserWindowY" of user defaults to windowY
end moved

on will open theObject
	--Get window position from defaults
	set windowX to contents of default entry "defaultBrowserWindowX" of user defaults
	set windowY to contents of default entry "defaultBrowserWindowY" of user defaults
	--Set window position if it's been saved at least once
	if windowX  "Testing" and windowY  "Testing" then
		set position of browserWindow to {windowX, windowY}
	end if
end will open
on choose menu item theObject
	(*Add your script here.*)
end choose menu item
