-- encode.applescript
-- iGP32

--  Created by Kounch on Tue Jul 13 2004.
--  Copyright (c) 2003 Kounch. All rights reserved.

-- This file is part of iGP32.
-- 
--  iGP32 is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.
--
--  iGP32 is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with iGP32; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

global myPath --Path to .app folder
global encoderWindow
global progressPanel

global codecstring
global audiostring
global pid --used to store the PID of long processes (encode video and audio)

--Several file paths
property origMovie : ""
property destMovie : ""
property tmpmovie : "/tmp/igp32.tmpvideo"
property tmpaudio : "/tmp/igp32.tmpaudio"
property tmplog : "/tmp/igp32.tmplog"

--Global variables which control loop encoding status
property encoding : false
property fin : false

--Global variables to handle error status and description
property fatalError : false
property errorText : ""

property encodingDelay : 0.5 --Delay factor before and between checking encoding status iterations

--Event received when clicked on cancel button, encode button (of main window or encoder window) and select file buttons
on clicked theObject
	if the name of theObject is "cancel" then
		--clicked on cancel button of progress panel
		try
			--kill actual encoding process
			theResult to do shell script "kill -9 " & pid
		end try
		--Set variables so that encoding finishes
		set fatalError to true
		set fin to true
		if errorText = "" then
			set errorText to localized string "ERR_CANCEL_KEY" from table "Localized"
		end if
		--finished cancel button handling
	else if the name of theObject is "encode" then
		--clicked on encoding button of main window
		if not encoding then
			encodewindow()
		end if
		--finished main window encoding button handling
	else if name of theObject is "startencode" then
		--clicked on encoding button of encoder window
		beginencoding()
		--finished encoder window encoding button handling
	else if name of theObject is "setfilename" then
		--clicked on set button for original video file
		
		--Initialize and displayopen file dialog
		set can choose directories of open panel to false
		set can choose files of open panel to true
		set selectTitle to localized string "SELECT_TITLE_KEY" from table "Localized"
		set title of open panel to selectTitle
		set selectButton to localized string "SELECT_BUTTON_KEY" from table "Localized"
		set prompt of open panel to selectButton
		set withResult to display open panel for file types {"avi", "mov", "mpg", "mpeg", "wmv", "asf", "vob", "dv"}
		--Check dialog result
		if withResult is 1 then
			--A file has been selected
			set origMovie to (path names of open panel as list) as string
			--Update window with info from selected file(s)
			tell encoderWindow
				set AppleScript's text item delimiters to {"/"}
				set contents of text field "filename" to last text item of origMovie
				set contents of text field "destfilename" to last text item of destMovie
			end tell
			--Obtain file size
			try
				--Call mencoder
				set theResult to do shell script myPath & "mencoder -v \"" & origMovie & "\" > " & tmplog
				--on error
				--No error handling as this command always returns an error
			end try
			--Filter mencoder dump to obtain file size
			set AppleScript's text item delimiters to {" "}
			set fileSize to do shell script "cat " & tmplog & " | grep 'File size'"
			set fileSize to words 4 thru 5 of fileSize
			--Update window info
			tell encoderWindow
				set contents of text field "filesize" to (fileSize as string)
			end tell
		end if
		--Finished with set button for original video file
	else if name of theObject is "setdestfilename" then
		--clicked on set button for destination video file
		
		--Initialize and show save file dialog
		set required file type of save panel to "avi"
		set selectTitle to localized string "SAVE_TITLE_KEY" from table "Localized"
		set title of save panel to selectTitle
		set selectButton to localized string "SAVE_BUTTON_KEY" from table "Localized"
		set prompt of save panel to selectButton
		set withResult to display save panel
		--Check dialog result
		if withResult is 1 then
			--A file name has been selected
			set destMovie to (path name of save panel as list) as string
			--Update window
			tell encoderWindow
				set AppleScript's text item delimiters to {"/"}
				set contents of text field "destfilename" to last text item of destMovie
			end tell
		end if
		--Finished with set button for destination file
	end if
end clicked


--Handler to remember window position
on moved theObject
	--Get window coordinates
	set {windowX, windowY} to position of theObject
	--Save coordinates to defaults
	set contents of default entry "defaultEncoderWindowX" of user defaults to windowX
	set contents of default entry "defaultEncoderWindowY" of user defaults to windowY
end moved

on will open theObject
	--Get window position from defaults
	set windowX to contents of default entry "defaultEncoderWindowX" of user defaults
	set windowY to contents of default entry "defaultEncoderWindowY" of user defaults
	--Set window position if it's been saved at least once
	if windowX  "Testing" and windowY  "Testing" then
		set position of encoderWindow to {windowX, windowY}
	end if
end will open


-- encoding window management routine
on encodewindow()
	--Get .app path plus Resources folder
	set myPath to ((POSIX path of (path to me)) as string) & "/Contents/Resources/"
	--Initialize window names
	set encoderWindow to window "encoder"
	set progressPanel to window "progress"
	--Show the main encoding window
	show encoderWindow
end encodewindow


-- Main video encoding routine 
on beginencoding()
	if origMovie is equal to "" then
		--original file not set
		tell button "setfilename" of encoderWindow to perform action
	else
		if (destMovie is equal to "") then
			--destination file not set
			tell button "setdestfilename" of encoderWindow to perform action
		else
			--Set value of global variable to check unrecoverable error inside the full encoding process
			set fatalError to false
			--Obtain desired movie quality and codec
			tell encoderWindow
				set fixAviHeader to state of button "FixAVI"
			end tell
			
			--Initialize progress panel
			tell progressPanel
				set AppleScript's text item delimiters to {"/"}
				set contents of text field "filename" to last text item of destMovie
				set progressText to localized string "BEGIN_ENCODING_KEY" from table "Localized"
				set contents of text field "progresstext" to progressText
				set contents of text field "remainingText" to " "
				set indeterminate of progress indicator "progress" to true
				start progress indicator "progress" of progressPanel
			end tell
			
			--Show progress panel
			display progressPanel attached to encoderWindow
			
			set encoding to true
			
			set errorText to ""
			--Obtain original movie size
			getMovieSize()
			if fixAviHeader is 1 then
				--Compress video
				encodeVideo()
				--Compress audio
				encodeaudio()
				--Mux AVI file
				muxAVI()
				--Remove temp files
				try
					do shell script "rm -f " & tmpmovie
					do shell script "rm -f " & tmpaudio
					do shell script "rm -f " & tmplog
				end try
			else
				--Display warning about fixing the final AVI file
				set warnText to localized string "WARN_KEY" from table "Localized"
				set warningText to localized string "WARN_AVI_HEADER" from table "Localized"
				set okText to localized string "OK_KEY" from table "Localized"
				set withResult to display alert warnText as warning message warningText default button okText
				
				encodeAVI()
				--Remove temp files
				try
					do shell script "rm -f " & tmplog
				end try
			end if
			
			--Close progress panel as there is no more encoding
			set contents of text field "remainingText" of progressPanel to " "
			stop progress indicator "progress" of progressPanel
			close panel (progressPanel)
			set encoding to false
			
			--Check if there were errors
			if fatalError then
				log "Encoding error: " & errorText
				--If there were errors, display dialog explaining what happened
				set errText to localized string "ERR_KEY" from table "Localized"
				set okText to localized string "OK_KEY" from table "Localized"
				set withResult to display alert errText as warning message errorText default button okText
			else
				set theSound to load sound "tatachan"
				play theSound
			end if
		end if
	end if
end beginencoding


--Sub which guesses the best video size (320x176 or 320x240) for encoding
on getMovieSize()
	try
		set theResult to do shell script myPath & "mencoder -v \"" & origMovie & "\" > " & tmplog
		---comented as this command always returns an error
		--on error
		--error wile processing shell video encoding
		--	set errorText to localized string "ERR_ANALYZE_KEY" from table "Localized"
		--	set fatalError to true
	end try
	
	if not fatalError then
		--Obtain video size and determine destination size from logfile
		try
			--filter mencoder output
			set theResult to ""
			set theResult to do shell script "cat " & tmplog & " | grep 'filefmt'"
			set AppleScript's text item delimiters to {""}
			set videoSize to word 7 of theResult
			set AppleScript's text item delimiters to {"x"}
			set videoX to text item 1 of videoSize
			set videoY to text item 2 of videoSize
			
			--get scaled y size for x size of 320
			set videoY to (320 * videoY / videoX)
			
			if videoY > 208 then
				--Aspect ratio like 4:9
				set videoY to 240
			else
				--Aspect ratio like 16:9
				set videoY to 176
			end if
			
			set videoX to 320
		on error
			log "VideoSize Error: " & theResult
			--error while evaluating video size
			set errorText to localized string "ERR_VIDEOSIZE_KEY" from table "Localized"
			set fatalError to true
		end try
		
		tell encoderWindow
			set quality to the contents of popup button "quality"
			set codec to the contents of popup button "codec"
		end tell
		
		--Set encoding bitrate parameters
		set vrate to 200 - (quality * 48)
		set frate to 14 - (quality * 2)
		--Encoding audio rate			
		set srate to 32 - (quality * 4)
		
		--Set codec parameters
		if codec = 0 then
			--selected xvid
			set codecstring to " -ovc xvid -xvidencopts bitrate=" & vrate & ":me_quality=6 "
		else if codec = 1 then
			--selected divx, actually not working :-( 
			--set codecstring to " -ovc lavc -lavcopts vcodec=mpeg4:vbitrate=" & vrate & ":vhq:vqmin=2:vqmax=20 "
			--set codecstring to " -ovc divx4 -divx4opts br=" & vrate & ":q=5:min_quant=2:max_quant=20 "
		end if
		
		set codecstring to " -vop scale=" & videoX & ":" & videoY & " -ofps " & frate & codecstring
		
		set audiostring to " -oac mp3lame -srate 11025 -lameopts vbr=3:br=" & srate & ":mode=3:vol=0 "
		
	end if
end getMovieSize


--Sub which encodes the video
on encodeVideo()
	if not fatalError then
		tell progressPanel
			--show information text for video encoding
			set progressText to localized string "VIDEO_ENCODING_KEY" from table "Localized"
			set contents of text field "progresstext" to progressText
			set indeterminate of progress indicator "progress" to false
			set content of progress indicator "progress" to 1
			set enabled of button "cancel" to true
		end tell
		
		--Video encoding parameters
		set encodingstring to codecstring & " -nosound "
		
		try
			--Shell command that sends to background the video encoding proccess 
			set pid to ""
			set pid to do shell script myPath & "mencoder -o \"" & tmpmovie & "\"" & encodingstring & "\"" & origMovie & "\" > " & tmplog & " 2>/dev/null & echo $!"
			
		on error
			log "mencoder Error: " & pid
			--error wile processing shell video encoding
			set fatalError to true
			set errorText to localized string "ERR_VIDEOENCODING_KEY" from table "Localized"
			tell button "cancel" of progressPanel to perform action
		end try
	end if
	
	--Continue if there were no errors before
	if not fatalError then
		--wait before checking encoding log. If not, we would get an error when trying to find 'Pos:'
		delay encodingDelay * 2
		--Loop control variable
		set fin to false
		repeat while not fin
			--wait before checking if encoding finished and updating the progress bar
			delay encodingDelay
			try
				--check encoding log
				set theResult to ""
				set theResult to do shell script "tail " & tmplog
				set theResult to (last paragraph of theResult) as string
				--check if process is running
				set theResult2 to ""
				set theResult2 to do shell script "ps -p " & pid
				set theResult2 to (last paragraph of theResult2) as string
				
				set AppleScript's text item delimiters to {""}
				if first word of theResult2 is "PID" then
					--wait before checking if encoding finished 
					delay encodingDelay * 4
					--The proccess has ended, we will check if it finished ok or not
					set AppleScript's text item delimiters to {":"}
					--Last line of encoding with mencoder begins with "Video stream:"
					if (text item 1 of theResult) is "Video stream" then
						set fin to true
					else
						log "Video encoding log parse Error:"
						log "Command: 'Video stream'  text item 1 of " & theResult
						set fatalError to true
						set errorText to localized string "ERR_VIDEOCRASH_KEY" from table "Localized"
						set fin to true
						--Cancel encoding process
						tell button "cancel" of progressPanel to perform action
					end if
				else
					set AppleScript's text item delimiters to {":"}
					--Encoding lines with percentage data begin with "Pos:"
					if (text item 1 of theResult) is "Pos" then
						--check video encoding log percentage
						set AppleScript's text item delimiters to {""}
						set theResult to word 4 of theResult
						set AppleScript's text item delimiters to {"%"}
						set theResult to text item 1 of theResult
						
						tell progressPanel
							--update progress bar
							set content of progress indicator "progress" to 1 + (theResult * 0.9)
						end tell
					end if
				end if
			on error
				log "Encoding log check Error:"
				log "Command 1: " & theResult
				log "Command 2: " & theResult2
				--error while checking video encoding log
				set fatalError to true
				set errorText to localized string "ERR_VIDEOLOG_KEY" from table "Localized"
				set fin to true
				--Cancel encoding process
				tell button "cancel" of progressPanel to perform action
			end try
		end repeat
	end if
end encodeVideo


--Sub which encodes the audio
on encodeaudio()
	if not fatalError then
		tell progressPanel
			--update progress text
			set progressText to localized string "AUDIO_ENCODING_KEY" from table "Localized"
			set contents of text field "progresstext" to progressText
		end tell
		
		set encodingstring to audiostring & " -ovc frameno "
	end if
	
	if not fatalError then
		try
			--Shell command that sends to background the audio encoding proccess
			set pid to ""
			set pid to do shell script myPath & "mencoder -o \"" & tmpaudio & "\"" & encodingstring & "\"" & origMovie & "\" >  " & tmplog & " 2>/dev/null & echo $!"
			
		on error
			log "mencoder Error: " & pid
			--error wile processing shell audio encoding
			set fatalError to true
			set errorText to localized string "ERR_AUDIOENCODING_KEY" from table "Localized"
			tell button "cancel" of progressPanel to perform action
		end try
	end if
	
	if not fatalError then
		--Loop control variable
		set fin to false
		--wait before checking encoding log. If not, we would get an error when trying to find 'Pos:'
		delay encodingDelay * 2
		repeat while not fin
			--wait before checking if encoding finished and updating the progress bar
			delay encodingDelay
			try
				--check in encoding log if the proccess has finished
				set theResult to ""
				set theResult to do shell script "tail " & tmplog
				set theResult to (last paragraph of theResult) as string
				--check if process is running
				set theResult2 to ""
				set theResult2 to do shell script "ps -p " & pid
				set theResult2 to (last paragraph of theResult2) as string
				
				set AppleScript's text item delimiters to {""}
				if first word of theResult2 is "PID" then
					--wait before checking if encoding finished
					delay encodingDelay * 4
					--The proccess has ended, we will check if it finished ok or not
					set AppleScript's text item delimiters to {":"}
					--Last line of encoding with mencoder begins with "Audio stream:"
					if (text item 1 of theResult) is "Audio stream" then
						set fin to true
					else
						log "Audio encoding log parse Error:"
						log "Command: 'Audio stream'  text item 1 of " & theResult
						set fatalError to true
						set errorText to localized string "ERR_AUDIOCRASH_KEY" from table "Localized"
						set fin to true
						--Cancel encoding process
						tell button "cancel" of progressPanel to perform action
					end if
				else
					set AppleScript's text item delimiters to {":"}
					--Encoding lines with percentage data begin with "Pos:"
					
					if (text item 1 of theResult) is "Pos" then
						--check audio encoding log percentage
						set AppleScript's text item delimiters to {""}
						set theResult to word 4 of theResult
						set AppleScript's text item delimiters to {"%"}
						set theResult to text item 1 of theResult
						
						tell progressPanel
							--update progress bar
							set content of progress indicator "progress" to 91 + (theResult * 0.08)
						end tell
					end if
				end if
			on error
				log "Audio encoding log check Error:"
				log "Command 1: " & theResult
				log "Command 2: " & theResult
				--error while checking audio encoding log
				set errorText to localized string "ERR_AUDIOLOG_KEY" from table "Localized"
				set fatalError to true
				set fin to true
			end try
		end repeat
	end if
end encodeaudio


--Sub which muxes AVI file
on muxAVI()
	tell progressPanel
		--update progress bar
		set content of progress indicator "progress" to 100
		set indeterminate of progress indicator "progress" to true
		--show Finished progress text
		set progressText to localized string "FINISHED_ENCODING_KEY" from table "Localized"
		set contents of text field "progresstext" to progressText
		--disable cancel button
		set enabled of button "cancel" to false
	end tell
	
	--Multiplex Final video file
	set encodingstring to " -vcodec copy -map 0.0:0.0 -acodec copy -map 1.1:0.1 "
	
	if not fatalError then
		try
			--multiplex video and audio files into final avi
			set theResult to ""
			set theResult to do shell script myPath & "ffmpeg -y -i \"" & tmpmovie & "\" -i \"" & tmpaudio & "\"" & encodingstring & "\"" & destMovie & "\" >  " & tmplog & " 2>&1"
		on error
			log "ffmpeg Error: " & theResult
			--error while multiplexing avi file
			set fatalError to true
			set errorText to localized string "ERR_MUXING_KEY" from table "Localized"
		end try
	end if
end muxAVI


--Sub which encodes an AVI without trying to fix the AVI Header
on encodeAVI()
	if not fatalError then
		tell progressPanel
			--show information text for video encoding
			set progressText to localized string "AVI_ENCODING_KEY" from table "Localized"
			set contents of text field "progresstext" to progressText
			set contents of text field "remainingText" to " "
			set indeterminate of progress indicator "progress" to false
			set content of progress indicator "progress" to 1
			set enabled of button "cancel" to true
		end tell
		
		--AVI encoding parameters
		set encodingstring to codecstring & audiostring
		
		try
			--Shell command that sends to background the video encoding proccess 
			set pid to ""
			set pid to do shell script myPath & "mencoder -o \"" & destMovie & "\"" & encodingstring & "\"" & origMovie & "\" > " & tmplog & " 2>/dev/null & echo $!"
		on error
			log "mencoder Error: " & pid
			--error wile processing shell video encoding
			set fatalError to true
			set errorText to localized string "ERR_AVIENCODING_KEY" from table "Localized"
			tell button "cancel" of progressPanel to perform action
		end try
	end if
	
	--Continue if there were no errors before
	if not fatalError then
		--wait before checking encoding log. If not, we would get an error when trying to find 'Pos:'
		delay encodingDelay * 2
		--Loop control variable
		set fin to false
		repeat while not fin
			--wait before checking if encoding finished and updating the progress bar
			delay encodingDelay
			--
			try
				--check encoding log
				set theResult to ""
				set theResult to do shell script "tail " & tmplog
				set theResult to (last paragraph of theResult) as string
				--check if process is running
				set theResult2 to ""
				set theResult2 to do shell script "ps -p " & pid
				set theResult2 to (last paragraph of theResult2) as string
				
				set AppleScript's text item delimiters to {""}
				if first word of theResult2 is "PID" then
					--wait before checking if encoding finished
					delay encodingDelay * 4
					--The proccess has ended, we will check if it finished ok or not
					set AppleScript's text item delimiters to {":"}
					--Last line of encoding with mencoder begins with "Video stream:"
					if (text item 1 of theResult) is "Audio stream" then
						set fin to true
					else
						log "AVI encoding log parse Error:"
						log "Command: 'Audio stream'  text item 1 of" & theResult
						set fatalError to true
						set errorText to localized string "ERR_VIDEOCRASH_KEY" from table "Localized"
						set fin to true
						--Cancel encoding process
						tell button "cancel" of progressPanel to perform action
					end if
				else
					set AppleScript's text item delimiters to {":"}
					--Encoding lines with percentage data begin with "Pos:"
					if (text item 1 of theResult) is "Pos" then
						--check video encoding log percentage and remaining time
						set tiempo to (text item 3 of theResult) as string
						set AppleScript's text item delimiters to {""}
						set theResult to word 4 of theResult
						set tiempo to word 1 of tiempo
						set AppleScript's text item delimiters to {"%"}
						set theResult to text item 1 of theResult
						set AppleScript's text item delimiters to {"min"}
						set tiempo to text item 1 of tiempo
						set tiempo to tiempo + 1
						tell progressPanel
							--update progress bar
							set content of progress indicator "progress" to 1 + (theResult * 0.99)
							set remainText to localized string "REMAINING_KEY" from table "Localized"
							if (tiempo > 1) then
								set remainTime to localized string "REMAINTIMESEVERAL_KEY" from table "Localized"
							else
								set remainTime to localized string "REMAINTIMEONE_KEY" from table "Localized"
							end if
							set contents of text field "remainingText" to remainText & tiempo & remainTime
						end tell
					end if
				end if
			on error
				log "AVI encoding log check Error:"
				log "Command 1: " & theResult
				log "Command 2: " & theResult
				--error while checking video encoding log
				set fatalError to true
				set errorText to localized string "ERR_VIDEOLOG_KEY" from table "Localized"
				set fin to true
				--Cancel encoding process
				tell button "cancel" of progressPanel to perform action
			end try
		end repeat
	end if
end encodeAVI

