-- maclink.applescript
-- iGP32

--  Created by Kounch on Mon 11/10/04.
--  Copyright (c) 2004 Kounch. All rights reserved.

-- This file is part of iGP32.
-- 
--  iGP32 is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.
--
--  iGP32 is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with iGP32; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

global mainWindow
global progressPanel
global maclinkPath
global USBDelay

global PackageAction
global ExecutableAction
global fatalError

--Handler that initializes global variables for this script, must be called right after using load script
on initLib()
	--Define USB wait time
	set USBDelay to 0.5
	--Define windows
	set mainWindow to window "main"
	set progressPanel to window "progressPanel"
	--Get path to maclink shell executable
	set maclinkPath to ((POSIX path of (path to me)) as string) & "/Contents/Resources/maclink"
	--set maclinkPath to "echo $1 >> /log.txt < "
end initLib

(*The characters that are valid for an 8.3-compliant file or subdirectory name include any combination of letters (A-Z) and/or numbers (0-9), plus the following special characters: 

    $    Dollar sign
    %    Percent sign
    '    Apostrophe
    `    Opening single quotation mark
    -    Hyphen
    @    At sign
    {    Left brace
    }    Right brace
    ~    Tilde
    !    Exclamation point
    #    Number sign
    (    Opening parenthesis
    )    Closing parenthesis
    &    Ampersand
    _    Underscore
    ^    Caret		
*)
--Handler called to check if a filename has 8.3 format
on checkFileName(theFile)
	set nameOK to true
	set validChars to "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789$%'`-@{}~!#()&_^"
	
	--DOS file names cannot have more than eight characters (and) an extension with no more than three chracters
	--Check length of fileName
	if (count of characters of theFile) > 12 then set nameOK to false
	--check number of dotted limited parts of fileName
	set AppleScript's text item delimiters to {"."}
	if (count of text items of theFile) > 2 then set nameOK to false
	--check number of characters of filename without extension
	set fileName to first text item of theFile
	if (count of characters of fileName) > 8 then set nameOK to false
	set fileXtension to last text item of theFile
	--check number of characters of extension
	if (count of characters of fileXtension) > 3 then set nameOK to false
	--check that name and extension are not empty
	if last character of theFile = "." or first character of theFile = "." then set nameOK to false
	
	--Check chars of fileName
	if (count of characters of fileName) > 0 then
		repeat with currentChar in (every character of fileName)
			set failure to true
			repeat with currentValidChar in (every character of validChars)
				if (currentChar as string) is equal to (currentValidChar as string) then set failure to false
			end repeat
			if failure is true then set nameOK to false
		end repeat
	end if
	
	--Check chars of file extension
	if (count of characters of fileXtension) > 0 then
		repeat with currentChar in (every character of fileXtension)
			set failure to true
			repeat with currentValidChar in (every character of validChars)
				if (currentChar as string) is equal to (currentValidChar as string) then set failure to false
			end repeat
			if failure is true then set nameOK to false
		end repeat
	end if
	
	return nameOK
end checkFileName


---Handler called to batch proccess FXE, GXB, ZPK and FPK files
on proccessFiles(theObject)
	--display progress panel
	display progressPanel attached to mainWindow
	
	--Check with every file in the list of files
	set fatalError to false
	repeat with theFile in theObject
		--process the object if there are no errors
		if not fatalError then
			--Get filename
			set savedDelimiters to AppleScript's text item delimiters
			set AppleScript's text item delimiters to {":"}
			set fileName to last text item of (theFile as string)
			set AppleScript's text item delimiters to savedDelimiters
			
			tell progressPanel
				set contents of text field "fileName" to fileName
				set actionText to localized string "PROCCESS_KEY" from table "Localized"
				set contents of text field "action" to actionText
				start progress indicator "progress"
				update
			end tell
			
			--Get name of each file
			set iPath to POSIX path of (theFile as string)
			--Get type of file
			set fileType to GetFileType(iPath)
			
			--Process files if there have been no errors
			if fileType is "FPK Package" or fileType is "ZPK Package" then
				managePackage(iPath, fileName)
			else if fileType is "FXE Executable" or fileType is "GXB Executable" then
				manageExecutable(iPath, fileName)
			end if
		end if
	end repeat
	--If there was an error the panel is already closed
	if not fatalError then
		close panel (progressPanel)
	end if
end proccessFiles


--Determine the kind of file
on GetFileType(theFile)
	--Get file extension
	set savedDelimiters to AppleScript's text item delimiters
	set AppleScript's text item delimiters to {"."}
	set fileXtension to last text item of theFile as string
	set AppleScript's text item delimiters to savedDelimiters
	
	--Determine file by extension
	if fileXtension is "zpk" then
		return "ZPK Package"
	else if fileXtension is "fpk" then
		return "FPK Package"
	else if fileXtension is "gxb" then
		return "GXB Executable"
	else if fileXtension is "fxe" then
		return "FXE Executable"
	else
		return "Other"
	end if
end GetFileType


--Deal with an installation package
on managePackage(theFile, fileName)
	if PackageAction is "Nothing" then
		--Ask what to do with  package
		set electionQuestion to fileName
		set electionText to localized string "PACKAGE_CHOOSE_KEY" from table "Localized"
		set installButton to localized string "INSTALL_BUTTON_KEY" from table "Localized"
		set decompressButton to localized string "DECOMPRESS_BUTTON_KEY" from table "Localized"
		set cancelButton to localized string "CANCEL_KEY" from table "Localized"
		
		set theReply to display alert electionQuestion message electionText default button installButton alternate button cancelButton other button decompressButton
		
		set chosenButton to button returned of theReply
		if chosenButton = installButton then
			--install file to GP32
			installPackage(theFile)
		else if chosenButton = decompressButton then
			--decompress file
			decompressPackage(theFile)
		end if
	else if PackageAction is "InstallAll" then
		--Install packages automatically
		installPackage(theFile)
	else if PackageAction is "DecompressAll" then
		--decompress files
		decompressPackage(theFile)
	else
		UnexpectedError("managePackage")
	end if
end managePackage

--Deal with an executable file
on manageExecutable(theFile, fileName)
	
	--Check filename format
	set validName to checkFileName(fileName)
	if validName is true then
		--Get file extension
		set fileType to GetFileType(theFile)
		
		if ExecutableAction is "Nothing" then
			--Ask what to do with executable
			set electionQuestion to fileName
			set electionText to localized string "EXECUTABLE_CHOOSE_KEY" from table "Localized"
			set copyButton to localized string "COPY_BUTTON_KEY" from table "Localized"
			set executeButton to localized string "EXECUTE_BUTTON_KEY" from table "Localized"
			set cancelButton to localized string "CANCEL_KEY" from table "Localized"
			
			if fileType is "GXB Executable" then
				set theReply to display alert electionQuestion message electionText default button copyButton alternate button cancelButton other button executeButton
			else
				set electionText to localized string "EXECUTABLE_COPY_KEY" from table "Localized"
				set theReply to display alert electionQuestion message electionText default button copyButton alternate button cancelButton
			end if
			
			set chosenButton to button returned of theReply
			if chosenButton = copyButton then
				---copy file to GP32
				copyToGP(theFile)
			else if chosenButton = executeButton then
				--execute file
				executeOnGP(theFile)
			end if
		else if ExecutableAction is "ExecuteAll" then
			--Execute files automatically
			executeOnGP(theFile)
		else if ExecutableAction is "CopyAll" then
			--Copy files automatically
			copyToGP(theFile)
		else
			UnexpectedError("manageExecutable")
		end if
	else
		FileNameError()
	end if
end manageExecutable


--Handler called when wishing to install a package to the console
on installPackage(theFile)
	--Update panel
	tell progressPanel
		set actionText to localized string "INSTALL_KEY" from table "Localized"
		set contents of text field "action" to actionText
		update
	end tell
	
	--Try to install file to GP32
	try
		delay USBDelay
		set theResult to ""
		set theResult to do shell script maclinkPath & " install -x " & installFile
		
	on error
		log "maclink install error: " & theResult
		--close panel
		close panel (progressPanel)
		--If we get in here, there has been some problem with USB connection
		USBError()
		set fatalError to true
	end try
end installPackage


--handler called when wanting to expand a package
on decompressPackage(theFile)
	--Get file extension
	set fileType to GetFileType(theFile)
	
	--Get Resources path
	set expPath to ((POSIX path of (path to me)) as string) & "/Contents/Resources/"
	
	--Prepare decompression dir
	set folderString to localized string "FOLDER_KEY" from table "Localized"
	set destDir to (theFile as string) & folderString
	try
		do shell script "mkdir \"" & destDir & "\""
	end try
	
	---decompress installation package
	if fileType is "ZPK Package" then
		--decompress zpk		
		set expPath to expPath & "upzpk \"" & theFile & "\" \"" & destDir & "\""
	else if fileType is "FPK Package" then
		--decompress fpk
		set expPath to expPath & "gppk -x -b \"" & destDir & "\" \"" & theFile & "\""
	else
		--oh oh, something went wrong
		return
	end if
	
	--Try to expand compressed file
	try
		set theResult to ""
		set theResult to do shell script expPath as string
		
	on error errText number errNumber
		log "Expand error: " & theResult
		--close panel
		close panel (progressPanel)
		--tell the user there was an error
		set errorText to localized string "ERR_KEY" from table "Localized"
		set errString to localized string "ERR_EXPAND_KEY" from table "Localized"
		set okText to localized string "OK_KEY" from table "Localized"
		display alert errorText as critical message errString default button okText attached to mainWindow
		set fatalError to true
	end try
end decompressPackage


--Handler called when wishing to copy a file to the console
on copyToGP(theFile)
	--Get file extension
	set fileType to GetFileType(theFile)
	
	--copy file to gp32
	if fileType is "FXE Executable" then
		--copy fxe to gp:/gpmm
		set destPath to "/GPMM/"
	else if fileType is "GXB Executable" then
		--copy gxb to gp:/game
		set destPath to "/GAME/"
	else
		return
	end if
	
	Upload(theFile, destPath)
end copyToGP

--Handler called to upload theItem file to the GP32
on Upload(theItem, destPath)
	try
		--Upload file to gp32 device
		delay USBDelay
		set theResult to ""
		set theResult to do shell script maclinkPath & " ul \"" & (destPath as string) & "\" \"" & (theItem as string) & "\""
	on error
		log "maclink ul error: " & theResult
		--close panel
		close panel (progressPanel)
		--If we get in here, there has been some problem with USB connection or the browser window
		USBError()
		set fatalError to true
	end try
end Upload

--handler called when wanting to execute a file directly (only GXB)
on executeOnGP(theFile)
	--Get file extension
	set fileType to GetFileType(theFile)
	
	--execute file in gp32
	if fileType is "GXB Executable" then
		--execute gxb in console
		try
			delay USBDelay
			set theResult to ""
			set theResult to do shell script maclinkPath & " run \" \"" & (theFile as string) & "\""
		on error
			log "maclink run error: " & theResult
			--close panel
			close panel (progressPanel)
			--If we get in here, there has been some problem with USB connection or the browser window
			USBError()
			set fatalError to true
		end try
		
	end if
end executeOnGP


--Handler which displays an error when the filename is not in 8.3 format
on FileNameError()
	set errorText to localized string "ERR_KEY" from table "Localized"
	set errDOSText to localized string "ERR_DOSFILE_KEY" from table "Localized"
	set okText to localized string "OK_KEY" from table "Localized"
	display alert errorText as warning message errDOSText default button okText
end FileNameError


--Handler called when we want to display an unexpected Error panel
on UnexpectedError(moduleName)
	set errorText to localized string "ERR_KEY" from table "Localized"
	set errDescription to localized string "ERR_UNEXPECTED_KEY" from table "Localized"
	set okText to localized string "OK_KEY" from table "Localized"
	display alert errorText as critical message (errDescription & moduleName) default button okText
end UnexpectedError


--Handler called when we want to display an USB Error panel
on USBError()
	set errorText to localized string "ERR_KEY" from table "Localized"
	set errUSB to localized string "ERR_USB_KEY" from table "Localized"
	set okText to localized string "OK_KEY" from table "Localized"
	display alert errorText as critical message errUSB default button okText attached to mainWindow
end USBError